@@ -318,6 +318,14 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         vlan_settings.OperationMode = self._OPERATION_MODE_ACCESS
         self._jobutils.add_virt_feature(vlan_settings, port_alloc)
 
+        # TODO(claudiub): This will help solve the missing VLAN issue, but it
+        # comes with a performance cost. The root cause of the problem must
+        # be solved.
+        vlan_settings = self._get_vlan_setting_data_from_port_alloc(port_alloc)
+        if not vlan_settings:
+            raise exceptions.HyperVException(
+                _('Port VLAN not found: %s') % switch_port_name)
+
     def set_vswitch_port_vsid(self, vsid, switch_port_name):
         port_alloc, found = self._get_switch_port_allocation(switch_port_name)
         if not found:
@@ -343,6 +351,15 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         vsid_settings.VirtualSubnetId = vsid
         self._jobutils.add_virt_feature(vsid_settings, port_alloc)
 
+        # TODO(claudiub): This will help solve the missing VSID issue, but it
+        # comes with a performance cost. The root cause of the problem must
+        # be solved.
+        vsid_settings = self._get_security_setting_data_from_port_alloc(
+            port_alloc)
+        if not vsid_settings:
+            raise exceptions.HyperVException(
+                _('Port VSID not found: %s') % switch_port_name)
+
     def _get_vlan_setting_data_from_port_alloc(self, port_alloc):
         return self._get_setting_data_from_port_alloc(
             port_alloc, self._vlan_sds, self._PORT_VLAN_SET_DATA)
@@ -380,10 +397,10 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
 
     def _get_setting_data(self, class_name, element_name, create=True):
         element_name = element_name.replace("'", '"')
-        q = self._conn.query("SELECT * FROM %(class_name)s WHERE "
-                             "ElementName = '%(element_name)s'" %
-                             {"class_name": class_name,
-                              "element_name": element_name})
+        q = self._compat_conn.query("SELECT * FROM %(class_name)s WHERE "
+                                    "ElementName = '%(element_name)s'" %
+                                    {"class_name": class_name,
+                                     "element_name": element_name})
         data = self._get_first_item(q)
         found = data is not None
         if not data and create:
@@ -392,11 +409,11 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         return data, found
 
     def _get_default_setting_data(self, class_name):
-        return self._conn.query("SELECT * FROM %s WHERE InstanceID "
-                                "LIKE '%%\\Default'" % class_name)[0]
+        return self._compat_conn.query("SELECT * FROM %s WHERE InstanceID "
+                                       "LIKE '%%\\Default'" % class_name)[0]
 
     def _create_default_setting_data(self, class_name):
-        return getattr(self._conn, class_name).new()
+        return getattr(self._compat_conn, class_name).new()
 
     def _get_first_item(self, obj):
         if obj:
